@@ -1,6 +1,6 @@
 #!/bin/bash
 
-# Copyright (C) 2016, 2018  Stefan Vargyas
+# Copyright (C) 2016, 2018, 2021  Stefan Vargyas
 # 
 # This file is part of Trie-Gen.
 # 
@@ -338,13 +338,15 @@ gen-func()
     local q=""          # pass `-q|--equal-func=NAME' to 'trie' (--equal-func=NAME) 
     local u=""          # pass `-u|--unique-prefix' to 'trie' (--unique-prefix)
     local r="+"         # result type name (the default is 'result_t') (--result-type=NAME)
+    local s=""          # pass `-t|--node-type=string' to 'trie' when result type is '-' (--return-string)
     local v=""          # be verbose (--verbose)
+    local w=""          # pass `-g|--gen-type=wide' to 'trie' (--wide-code)
     local z=""          # pass `-z|--zero-start' to 'trie' (--zero-start)
 
     local opt
     local OPT
     local OPTN
-    local opts=":Cde:f:h:i:p:P:q:r:Ruvxz-:"
+    local opts=":Cde:f:h:i:p:P:q:r:Rsuvwxz-:"
     local OPTARG
     local OPTERR=0
     local OPTIND=1
@@ -361,8 +363,12 @@ gen-func()
                 opt='R' ;;
             error|func-name|home|input|result-type|prefix-func|unique-prefix|verbose|zero-start)
                 opt="${OPT:0:1}" ;;
+            return-string)
+                opt='s' ;;
             equal-func)
                 opt='q' ;;
+            wide-code)
+                opt='w' ;;
             *)	error --long -o
                 return 1
                 ;;
@@ -382,7 +388,7 @@ gen-func()
             [CR])
                 act="$opt"
                 ;;
-            [uvz])
+            [suvwz])
                 optopt
                 ;;
             [hi])
@@ -427,6 +433,7 @@ gen-func()
     shift $((OPTIND - 1))
 
     [ "$r" == '-' ] && r=''
+    [ -z "$r" -a -n "$s" ] && edef='NULL'
 
     local n
     local v2
@@ -445,9 +452,9 @@ gen-func()
 
     local h2
     local a
-    local s
     local s2
     local s3
+    local s4
     local c
     if [ "$act" == "R" ]; then
         a='
@@ -555,29 +562,37 @@ $l -T ternary -oe -ec -gc"
             }
         }
 
-        s='
-            1i\'
-        [ -z "$r" ] && s+='
-int '"$f"'(const char* p)\'
-        [ -n "$r" ] && s+='
-bool '"$f"'(const char* n, '"$r"'& t)\'
-        s+='
+        local r2
+        if [ -z "$r" ]; then
+            [ -n "$s" ] &&
+            r2='const char*' ||
+            r2='int'
+        else
+            r2='bool'
+        fi
+
+        s4="
+            1i\\
+$r2 $f(const char*"
+        [ -z "$r" ] && s4+=' p)\'
+        [ -n "$r" ] && s4+=' n, '"$r"'& t)\'
+        s4+='
 {'"${s3:+\\
     // pattern: $s3}"'
             $a}'
-        [ -n "$r" ] && s+='
+        [ -n "$r" ] && s4+='
             :0
             s/^(\t*)    /\1\t/
             t0
             s/(\*\s*)p\b/\1n/g'"${s2:+
 $s2}"
-        [ "$P" == 'f' -a -n "$r" ] && s+='
+        [ "$P" == 'f' -a -n "$r" ] && s4+='
             s/(if\s*\(!?[a-zA-Z_][0-9a-zA-Z_]*\()p\b(.*)$/\1n\2 {/g
             s/(,\s*)p(\)\))/\1n\2/g
             s/\bp(\s*\+=)/n\1/g'
-        [ -z "$r" -a "$e" != 'error' ] && s+='
+        [ -z "$r" -a "$e" != 'error' ] && s4+='
             s/(return\s+)error\b/\1'"$e"'/'
-        [ -n "$r" ] && s+='
+        [ -n "$r" ] && s4+='
             s/\*n\s*==\s*0\s*\)'"${u:++}"'$/\0 {/
             s/(return\s+)error\b/\1false/'
         # stev: no need to quote '$P', '$p' and '$q' below
@@ -585,12 +600,13 @@ $s2}"
         [ -n "$r" ] && c="\
 awk '$a' \\
 $t|"$'\n'
+        [ -n "$w" ] && w='w' || w='c'
         c+="\
-$l -T ternary -oc -gc${P:+ -P$P}${p:+ -p $p}${q:+ -q $q}${u:+ -u}${z:+ -z}"
-        [ -z "$r" ] && c+=" -ti${i:+ < \\
+$l -T ternary -oc -g$w${P:+ -P$P}${p:+ -p $p}${q:+ -q $q}${u:+ -u}${z:+ -z}"
+        [ -z "$r" ] && c+=" -t${s:-i}${i:+ < \\
 $i}"
         c+="|
-sed -r '$s'"
+sed -r '$s4'"
         [ -n "$r" ] && c+="|
 sed -r ':0;s/^( *)\t/\1    /;t0'"
 
