@@ -1,6 +1,6 @@
 #!/bin/bash
 
-# Copyright (C) 2016, 2018  Stefan Vargyas
+# Copyright (C) 2016, 2018, 2021  Stefan Vargyas
 # 
 # This file is part of Trie-Gen.
 # 
@@ -25,7 +25,11 @@ gcc-version()
     "$p" --version|
     sed -nr '
         1 {
-            s/^gcc.*\(.*\)\s+([0-9]+\.[0-9]+\.[0-9]+)(\s.*)?$/\1/p
+            /^gcc.*\s([0-9]+\.[0-9]+\.[0-9]+)(\s.*)?$/bp
+            /^gcc.*\(.*\)\s+([0-9]+\.[0-9]+\.[0-9]+)(\s.*)?$/bp
+            q
+            :p
+            s//\1/p
             q
         }'|
     awk -F. '{
